pub trait BlockConsumer:
Sync
+ Send
+ 'static {
// Required methods
fn wants_spent_utxos(&self) -> bool;
fn on_block(
&self,
block: &Block,
height: u32,
spent_utxos: Option<&HashMap<OutPoint, UtxoData>>,
);
}Expand description
Trait for components that need to receive notifications about new blocks.
Required Methods
Sourcefn wants_spent_utxos(&self) -> bool
fn wants_spent_utxos(&self) -> bool
Return true if this consumer wants the set of spent UTXOs.